/*
 * DeltaEditorData.java
 *
 * Created on 11. Februar 2004, 14:01
 */

package net.metadiversity.diversity.navigator.db.delta_editor;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class DeltaEditorData 
{
    static Logger logger = Logger.getLogger( DeltaEditorData.class.getName() ); 
    
    private int characterCount;
    private int itemNameCount;
    
    private boolean showItemIndex   = false;
    private boolean showHeaderIndex = false;
        
    boolean debug = false;
    
    // private String TABLE_PREFIX = "" + TABLE_PREFIX + "";
    private String TABLE_PREFIX = "LIASlight";
    
    
    java.util.Vector charNameVector     = null;
    java.util.Vector itemNameVector     = null;
    java.util.Vector inapplicableVector = null;
    java.util.Vector depVector          = null;
    java.sql.Connection con             = null;
    
    java.sql.PreparedStatement getCharacterDataStatment     = null;
    java.sql.PreparedStatement getCharacterStatesStatment   = null;
    
    String tableCHAR    = null;
    String tableITEM    = null;
    String tableDESCR   = null;
    String tableCS      = null;
    String tableDEP     = null;
    
    public static final int    DESCR_CS         = 1;
    public static final int    DESCR_MODIFIER   = 2;
    public static final int    DESCR_X          = 3;
    public static final int    DESCR_TXT        = 4;
    public static final int    DESCR_NOTES      = 5;
    public static final int    DESCR_SEQ        = 6;
    
    /** Creates a new instance of DeltaEditorData */
    public DeltaEditorData( java.sql.Connection con, String tablePrefix ) 
    {
        TABLE_PREFIX = tablePrefix;
        this.con = con;
        // init
        tableCHAR = "\"" + TABLE_PREFIX + "_CHAR\"";
        String characterNamesSql = "SELECT \"CID\", \"CharName\", \"Unit\", \"Type\" FROM " + tableCHAR + " ORDER BY \"CID\"; ";

        tableITEM = "\"" + TABLE_PREFIX + "_ITEM\"";
        String taxonNamesSql    = "SELECT \"IID\", \"ItemName\" FROM " + tableITEM + " ORDER BY \"ItemName\"; ";
                
        tableDESCR = "\"" + TABLE_PREFIX + "_DESCR\"";
        String taxonDescriptionSql = "SELECT \"CS\",\"Modifier\", \"X\", \"TXT\", \"Notes\", \"SEQ\" FROM " + tableDESCR + " WHERE \"IID\"=? AND \"CID\"=? ORDER BY \"CS\" ASC";
        
        tableCS = "\"" + TABLE_PREFIX + "_CS\"";
        tableDEP = "\"" + TABLE_PREFIX + "_DEP\"";
        
        String characterStatesSql = "SELECT \"CS\", \"CharStateName\" FROM " + tableCS + " WHERE \"CID\"=?  ORDER BY  \"StateID\"; ";
                
        
        charNameVector      = new java.util.Vector();
        itemNameVector      = new java.util.Vector();
        inapplicableVector  = new java.util.Vector();
        depVector           = new java.util.Vector();
        try
        {
            java.sql.Statement stmt = con.createStatement();
            //------------------------
            // init PreparedStatement 
            //------------------------
            getCharacterDataStatment    = con.prepareStatement( taxonDescriptionSql );
            getCharacterStatesStatment  = con.prepareStatement( characterStatesSql );
            //------------------
            // Character names
            //------------------
            java.sql.ResultSet rs   = stmt.executeQuery( characterNamesSql );
            int index = 0;
            while( rs.next() )
            {
                int     cid             = rs.getInt( "CID" );
                String  characterName   = rs.getString( "CharName" );
                String  unit            = rs.getString( "Unit" );
                String  type            = rs.getString( "Type" );
                
                charNameVector.add( index++, new HeadElement( cid,  characterName, unit, type ) );
            }
            //-------------
            // item names
            //-------------
            rs   = stmt.executeQuery( taxonNamesSql );
            index = 0;
            while( rs.next() )
            {
                int     iid             = rs.getInt( "IID" );
                String  itemName        = rs.getString( "ItemName" );
                
                itemNameVector.add( index++, new ItemElement( iid,  itemName ) );
            }
            //-----------------
            // inapplicalible
            //-----------------
            rs   = stmt.executeQuery( "SELECT DISTINCT \"InapplicableCID\" FROM " + tableDEP );
            index = 0;
            while( rs.next() )
            {
                String  inApp = rs.getString( 1 );                
                inapplicableVector.add( inApp );
            }
        }
        catch( java.sql.SQLException ex )
        {
            logger.error( ex.getMessage() );             
        }        
        // set size of row and columne
        characterCount = charNameVector.size();
        itemNameCount = itemNameVector.size();
        // init data for inapplicalible test
        initInapplicalibleVector();
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getItemNameCount()
    {
        return itemNameCount;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getCharacterCount()
    {
        return characterCount;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void showItemIndex( boolean b )
    {
        showItemIndex = b;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void showHeaderIndex( boolean b )
    {
        showHeaderIndex = b;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getIID( int row )
    {
        return (( ItemElement ) itemNameVector.get( row )).getIID(); // offest for HeadElement
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getCID( int col )
    {
        return (( HeadElement ) charNameVector.get( col )).getCID(); // offset for ItemElement
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public CharacterStateDescription getTableElement( int row, int col )
    {
        // logger.debug( "getTableElement( " + row + ", " + col + ")" );
        HeadElement hl = ( HeadElement ) charNameVector.get( col ); // offset for ItemElement
        ItemElement ie = ( ItemElement ) itemNameVector.get( row ); // offest for HeadElement
        
        int iid = ie.getIID();
        int cid = hl.getCID();
        
        java.lang.String characterType = hl.getType();
        
        boolean isInAppl = false; 

        try
        {
            //-------------------------
            // Inapplicabile 
            // ??? Not tested well ??? 
            //-------------------------                       
            if( inapplicableVector.contains( String.valueOf( cid ) ) )
            {   
                isInAppl = true;
                InapplicalibleTest it = new InapplicalibleTest( cid );
                
                String sql = "SELECT \"CID\", \"CS\" FROM " + tableDESCR + " WHERE \"IID\"=" + iid + " AND ( " + it.getWhereClause() + " )";                
                // logger.debug( sql );
                java.sql.Statement stm = con.createStatement();
                java.sql.ResultSet irs = stm.executeQuery( sql );
                while( irs.next() )
                {
                    // logger.debug( irs.getString( 1 ) + " " + irs.getString( 2 ) );
                    isInAppl = it.isInapplicalible( irs.getInt( 1 ), irs.getString( 2 ) );
                }
                //isInAppl = it.isInapplicalible();
            }            
            //------------------------
            //  get Character States
            //------------------------
            getCharacterDataStatment.setInt( 1, iid );
            getCharacterDataStatment.setInt( 2, cid );

            java.sql.ResultSet rs = getCharacterDataStatment.executeQuery();

            CharacterStateDescription csd = new CharacterStateDescription();
            csd.setInapplicalible( isInAppl );
            while( rs.next() )
            {
                csd.add( new CharacterStateDescriptionEntry( iid, cid, 
                                                             rs.getString( DESCR_MODIFIER ),
                                                             rs.getString( DESCR_CS ), 
                                                             rs.getDouble( DESCR_X ), 
                                                             rs.getString( DESCR_TXT ), 
                                                             rs.getString( DESCR_NOTES ), 
                                                             rs.getInt( DESCR_SEQ ),
                                                             characterType                                                             
                                                           ) 
                        );
            }
            return csd;        
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }
        logger.fatal( "Software error" );
        return null;        
    }
    //--------------------------------------------------------------------------
    // get IID
    //--------------------------------------------------------------------------    
    public int getItemIID( int row )
    {
        ItemElement ie = ( ItemElement ) itemNameVector.get( row ); // ??? offest for HeadElement
        return ie.getIID();        
    }
    //--------------------------------------------------------------------------
    // get Lable for this Item 
    //--------------------------------------------------------------------------
    public String getItemLableText( int row )
    {    
        // Show Taxon Name
        String str;
        ItemElement ie = ( ItemElement ) itemNameVector.get( row ); // ??? offest for HeadElement
        str = ( String ) ie.getItemName();
        if( showItemIndex )
        {
            boolean showIndex = false;  // false --> show line number

            java.text.NumberFormat nf = java.text.NumberFormat.getInstance();
            nf.setMinimumIntegerDigits( 3 );
            if( showIndex )
            {
                str = nf.format( ie.getIID() ) + " " + str;
            }
            else
            {
                // show line number
                String n = "" + row + "";                    
                while( n.length() < 4 )
                {
                    n = "0" + n;
                }
                str = n  + " " + str;
            }
        }
        return str;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public String getItemText( int row )
    {
        if( debug ) logger.debug( "getItemText( row = " + row + " )" );
            // Show Taxon Name
            String str;
            ItemElement ie = ( ItemElement ) itemNameVector.get( row ); // ??? offest for HeadElement
            str = ( String ) ie.getItemName();
            return str;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getHeadIndex( int col )
    {
        if( debug ) logger.debug( "getHeadIndex( col = " + col + " )" );
        
        if( charNameVector != null )
        {
            HeadElement he = ( HeadElement ) charNameVector.get( col ); // ??? offset for ItemElement
            
            String str = ( String ) he.getCharacterName();
            // str = str.replaceAll( " ", "<p>" );
            if( showHeaderIndex )
            {
                str = "<html>" + he.getCID() + "<p>" + str + "</html>";
            }
            else
            {
                str = "<html>" + str + "</html>";
            }            
            return str;
        }
        else
        {
            return "none";
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getHeadText( int col )
    {
        if( debug ) logger.debug( "getHeadText( col = " + col + " )" );
        
        String str;
        HeadElement hl = ( HeadElement ) charNameVector.get( col ); // ??? offset for ItemElement
        str = ( String ) hl.getCharacterName();
        // str = str.replaceAll( " ", "<p>" );
        // str = "<html>" + hl.getCID() + "<p>" + str + "</html>";
        str = "<html>" + str + "</html>";
        return str;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getTypeUnit( int col )
    {
        if( debug ) logger.debug( "getHeadTypUnit( col = " + col + " )" );
        
        String str;
        HeadElement hl = ( HeadElement ) charNameVector.get( col ); // ??? offset for ItemElement
        str = hl.getUnit();
        if( str == null )
        {
            return null;
        }
        str = ( String ) hl.getType() + ": " + str;
        return str;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    class HeadElement
    {
        int     cid;
        String  characterName;
        String  unit;
        String  type;
        
        public HeadElement( int cid, String characterName, String unit, String type )
        {
            this.cid = cid;
            this.characterName = characterName;
            this.unit = unit;
            this.type = type;
        }
        public int getCID()
        {
            return cid;
        }
        public String getCharacterName()
        {
            return characterName;
        }
        public String getType()
        {
            return type;
        }
        public String getUnit()
        {
            return unit;
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    class ItemElement
    {
        int     iid;
        String  name;
        
        public ItemElement( int iid, String itemName )
        {
            this.iid = iid;
            this.name = itemName;
        }
        public int getIID()
        {
            return iid;
        }
        public String getItemName()
        {
            return name;
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------        
    public java.util.Vector getCharacterStatesNames( int col )
    {
        if( debug ) logger.debug( "getCharacterStatesNames( col = " + col + " )" );
        
        HeadElement hl = ( HeadElement ) charNameVector.get( col ); // offset for ItemElement
        int cid = hl.getCID();
        
        if( debug ) logger.debug( "getCharacterStatesNames( cid = " + cid + " )" );
        try
        {
            getCharacterStatesStatment.setInt( 1, cid );
            java.sql.ResultSet rs = getCharacterStatesStatment.executeQuery();

            java.util.Vector result = new java.util.Vector();
            while( rs.next() )
            {
                result.add( new CharacterStateElement( rs.getString( 1 ), rs.getString( 2 ) ) );                                
            }
            return result;
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );
            return null;
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------        
    class Dep
    {
            public int cid;
            public String cs;
            public int dep;
            
            public Dep( int cid, String cs, int dep )
            {
                this.cid = cid; 
                this.cs = cs.trim(); 
                this.dep = dep;
            }
    };
    class Inapplicalible implements java.lang.Comparable
    {
            public int cid;
            public String cs;
            
            public Inapplicalible( int cid, String cs )
            {
                this.cid = cid; 
                this.cs = cs.trim(); 
            }
            
            public int compareTo( Object obj )
            {
                Inapplicalible comp = ( Inapplicalible ) obj;
                
                if( cid < comp.cid )
                    return -1;
                else if( cid > comp.cid )
                    return 1;
                else 
                    return cs.compareTo( comp.cs );
            }
            
            public boolean equals( Object obj )
            {
                Inapplicalible comp = ( Inapplicalible ) obj;
                
                return ( cid == comp.cid && cs.equals( comp.cs ) );
            }
            //------------------------------------------------------------------
            // implement hashCode(), this is used by HashSet to determinate
            // if to objects my be equals
            //------------------------------------------------------------------
            public int hashCode()
            {
                // int h = super.hashCode();
                // System.out.print( cid + " " + cs + " " + h );
                // h = cid  + cs.hashCode();
                // System.out.println( " " + h );
                return cid  + cs.hashCode();
            }
    };
    class InapplicalibleTest
    {
        boolean inapplicalibleFlag;
        java.util.HashSet inapplicalible = null;
        StringBuffer sb = null;
        java.util.HashSet usedCharacterStates = null;
        //----------------------------------------------------------------------
        public InapplicalibleTest( int cid )
        {
            inapplicalible  = new java.util.HashSet();
            usedCharacterStates = new java.util.HashSet();
            inapplicalibleFlag = true;
            testInapplicable( cid );
        }
        //----------------------------------------------------------------------
        private void addInapplicalible( Inapplicalible inappl )
        {
            inapplicalible.add( inappl );
        }
        //----------------------------------------------------------------------
        public boolean isInapplicalible()
        {            
            return inapplicalibleFlag;
        }
        //----------------------------------------------------------------------
        public boolean isInapplicalible( int cid, String cs )
        {            
            usedCharacterStates.remove( new Integer( cid ) );
            if( ! inapplicalible.contains( new Inapplicalible( cid, cs ) ) )
            {    
                inapplicalibleFlag = false;
            }
            if( usedCharacterStates.size() > 0 )
            {   
                return true;
            }
            return inapplicalibleFlag;
        }
        //----------------------------------------------------------------------
        public void testInapplicable( int inappl )
        {
            for( int i = 0; i < depVector.size(); i++ )
            {
                Dep dep = (Dep) depVector.get( i );

                if( dep.dep == inappl )
                {
                    // System.out.println( dep.cid + " " + dep.cs + " " + dep.dep );
                    usedCharacterStates.add( new Integer( dep.cid ) );                    
                    addInapplicalible( new Inapplicalible( dep.cid, dep.cs  ) );
                    testInapplicable( dep.cid );
                }
            }
        }
        //----------------------------------------------------------------------
        public String getWhereClause()
        {
            java.util.Iterator iter = usedCharacterStates.iterator();
            boolean firstFlag = true;
            sb = new StringBuffer();
            while( iter.hasNext() )
            {    
                if( ! firstFlag )
                {
                    sb.append( " OR " );
                }
                sb.append( "\"CID\"=" + (Integer) iter.next() );
                firstFlag = false;
            }
            return sb.toString();
        }
    };
    public void initInapplicalibleVector()
    {
        try
        {
            java.sql.Statement stmt = con.createStatement();
            java.sql.ResultSet rs   = stmt.executeQuery( "SELECT DISTINCT \"CID\", \"CS\", \"InapplicableCID\" FROM " + tableDEP + " ORDER BY \"InapplicableCID\", \"CID\", \"CS\" " );
            while( rs.next() )
            {
                int     cid     = rs.getInt( 1 );
                String  cs      = rs.getString( 2 );
                int     dep     = rs.getInt( 3 );                
                
                depVector.add( new Dep( cid, cs, dep ) );
            }
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }
    }    
    //--------------------------------------------------------------------------
    // Database add/update/delete CharacterDescriptions 
    // Table: TABLE_PREFIX + tableDESCR
    // PrimaryKey: 
    //--------------------------------------------------------------------------
    private String getDescrWhereClause( CharacterStateDescriptionEntry csde )
    {
        return " WHERE (\"IID\"=" + csde.getIID() + " AND \"CID\"=" + csde.getCID() + " AND \"CS\"='" + csde.getCS() + "')";
    }
    private String getDescrColumnNames( CharacterStateDescriptionEntry csde )
    {
        return " ( \"IID\",\"CID\",\"CS\",\"Modifier\", \"X\", \"TXT\", \"Notes\", \"SEQ\" ) ";
    }
    private String getDescrValues( CharacterStateDescriptionEntry csde )
    {
        return " VALUES ( " + csde.getIID() + ", " + csde.getCID() + ", '" + csde.getCS() + "', '" + csde.getModifier() + "', " + csde.getX() + ", '" + csde.getText() + "', '" + csde.getNotes() + "', " + csde.getSEQ() + " )";
    }
    private String getDescrSet( CharacterStateDescriptionEntry csde )
    {
        return " SET \"IID\"=" + csde.getIID() + ", \"CID\"=" + csde.getCID() + ", \"CS\"='" + csde.getCS() + "', \"Modifier\"='" + csde.getModifier() + "', \"X\"=" + csde.getX() + ", \"TXT\"='" + csde.getText() + "', \"Notes\"='" + csde.getNotes() + "', \"SEQ\"=" + csde.getSEQ() + " ";
    }
    //--------------------------------------------------------------------------
    public void inserCharacterStateDescriptionEntry( CharacterStateDescriptionEntry csde )
    {
        // add to datebase
        String sql = "INSERT INTO " + tableDESCR + getDescrColumnNames( csde ) + getDescrValues( csde );
        
        logger.debug( sql );
        executeUpdate( sql );
        // add to data
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void removeCharacterStateDescriptionEntry( CharacterStateDescriptionEntry csde )
    {
        // remove from datebase
        String sql = "DELETE FROM " + tableDESCR + getDescrWhereClause( csde );
        
        logger.debug( sql );
        executeUpdate( sql );
        // remove data
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void updateCharacterStateDescriptionEntry( CharacterStateDescriptionEntry csde )
    {
        // update datebase
        String sql = "UPDATE " + tableDESCR + getDescrSet( csde ) + getDescrWhereClause( csde );
        
        logger.debug( sql );
        executeUpdate( sql );
        // update data
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void executeUpdate( String sql )
    {
        try
        {
            java.sql.Statement stmt = con.createStatement();
            int rowCount = stmt.executeUpdate( sql );
            if( rowCount != 1 )
            {
                logger.fatal( "Update ERROR : " + sql );
            }
            stmt.close();
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.String getCharacterType( int cid )
    {
        java.util.ListIterator iter = charNameVector.listIterator();
        
        while( iter.hasNext() )
        {
            HeadElement hl = ( HeadElement ) iter.next();
            
            if( hl.cid == cid )
            {
                return hl.getType();
            }
        }
        return null;
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private int getNextItemID()
    {
        try
        {
            java.sql.Statement stmt = con.createStatement();
            java.sql.ResultSet rs   = stmt.executeQuery( "SELECT max( \"IID\" ) FROM " + tableITEM );  
            int iid = -2;
            if( rs.next() )
            {
                iid = rs.getInt( 1 );
            }                
            rs.close();
            stmt.close();            
            return ++iid;
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }        
        return -1;        
    }
    //--------------------------------------------------------------------------
    // return iid
    //--------------------------------------------------------------------------    
    public int insertNewItem( String itemName )
    {
        int iid = getNextItemID();        
        String insertNewItemSql = "INSERT INTO " + tableITEM + "( \"IID\", \"ItemName\" ) VALUES ( " + iid + ", '" + itemName + "' )";                  
        try
        {
            java.sql.Statement stmt = con.createStatement();
            logger.fatal( "Delta Editor INSERT: " + insertNewItemSql );
            if( stmt.execute( insertNewItemSql ) )
            {
                logger.fatal( "INSERT ERROR : " + insertNewItemSql );
            }
            stmt.close();
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }        
        return iid;
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------    
    public void deleteItem( String itemName )
    {
        String deleteItemSql = "DELETE FROM " + tableITEM + " WHERE \"ItemName\" = '" + itemName + "' ";                  
        try
        {
            java.sql.Statement stmt = con.createStatement();
            logger.fatal( "Delta Editor DELETE: " + deleteItemSql );
            if( stmt.execute( deleteItemSql ) )
            {
                logger.fatal( "DELETE ERROR : " + deleteItemSql );
            }
            stmt.close();
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }        
        return;
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------    
    public void renameItem( String newItemName, String oldItemName )
    {
        String renameItemSql = "UPDATE " + tableITEM + " SET \"ItemName\" = '" + newItemName + "' WHERE \"ItemName\" = '" + oldItemName + "' ";                  
        try
        {
            java.sql.Statement stmt = con.createStatement();
            logger.fatal( "Delta Editor RENAME: " + renameItemSql );
            if( stmt.execute( renameItemSql ) )
            {
                logger.fatal( "RENAME ERROR : " + renameItemSql );
            }
            stmt.close();
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }        
        return;
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------    
    public void insertCopyNewItem( String itemName, int iid )
    {
        try
        {
            int newIID = insertNewItem( itemName );            
            // copy item data                             
            String copySql = "SELECT \"CID\", \"Modifier\", \"CS\", \"X\", \"TXT\", \"Notes\", \"SEQ\" FROM " + tableDESCR + " WHERE \"IID\"=" + iid;
            java.sql.Statement stmt = con.createStatement();
            java.sql.ResultSet rs = stmt.executeQuery( copySql );
            java.util.Vector v = new java.util.Vector();            
            while( rs.next() )
            {
                v.add( new CharacterStateDescriptionEntry(   newIID,            // IID
                                                             rs.getInt(    1 ), // CID 
                                                             rs.getString( 2 ), // Modifier
                                                             rs.getString( 3 ), // CS
                                                             rs.getDouble( 4 ), // X
                                                             rs.getString( 5 ), // TXT
                                                             rs.getString( 6 ), // Notes
                                                             rs.getInt(    7 ), // SEQ
                                                             ""     // characterType                                                 
                                                           ) 
                        );
            }
            rs.close();
            stmt.close();
            java.util.ListIterator iter = v.listIterator();            
            while( iter.hasNext() )
            {
                CharacterStateDescriptionEntry csde = (CharacterStateDescriptionEntry) iter.next();
                inserCharacterStateDescriptionEntry( csde );
            }
        }
        catch( java.sql.SQLException ex )
        {
            logger.fatal( ex.getMessage() );             
        }                
    }
    //--------------------------------------------------------------------------
    // Search ItemName
    //--------------------------------------------------------------------------
    public int getRowByItemName( String name )
    {
        java.util.ListIterator iter = itemNameVector.listIterator();
        
        while( iter.hasNext() )
        {
            ItemElement item = ( ItemElement ) iter.next();
            
            if( item.getItemName().startsWith( name ) )
            {
                System.out.println( "Found: " + item.getItemName() );
                return iter.previousIndex();
            }           
        }
        return -1;
    }
}
