/*
 * DeltaEditor.java
 *
 * Created on 9. Februar 2004, 12:19
 */


package net.metadiversity.diversity.navigator.gui.delta_editor;

import org.apache.log4j.Logger;
import org.apache.log4j.Priority;
import org.apache.log4j.Level;

import java.awt.Dimension;
import java.awt.Rectangle;

import net.metadiversity.diversity.navigator.gui.DnOptionPane;
import net.metadiversity.diversity.navigator.gui.DnMenuItem;
import net.metadiversity.diversity.navigator.db.delta_editor.DeltaEditorData;
import net.metadiversity.diversity.navigator.db.delta_editor.CharacterStateDescription;
/**
 *
 * @author  didi
 */
/*
 * to speed up the data access we neet an index:
 *
 * CREATE INDEX "LIASlight_DESCR_idx" ON "LIASlight_DESCR" ("IID");
 */

public class DeltaEditor extends javax.swing.JPanel implements java.awt.event.ActionListener
{
    static Logger logger = Logger.getLogger( DeltaEditor.class.getName() ); 
   
    // ??? public
    public  CharacterStateEditor    characterEditorPane;
    private CharacterStateSelector  characterStatesPane;
    
    private javax.swing.JSplitPane  editSplitPane;
    private javax.swing.JTable      editorTable;
    private javax.swing.JTable      rowHeaderTable;
    private javax.swing.JScrollPane editorTableScrollPane;
    private javax.swing.JSplitPane  mainSplitPane;    
    private DeltaEditorData         ded;
    
    private java.sql.Connection     con;
    
    private final static int ROW_HEADER_MIN_WIDTH = 240;
    private final static int COLUMN_MIN_WIDTH     = 60;
    private final static int SELECTION_WIDTH      = 388;
    private final static boolean debug = false;
    
    private DeltaEditorChangeListener deltaEditorChangeListener;
    
    private String tablePrefix = "LIASlight";
    
    /** Creates new form DeltaEditor */
    public DeltaEditor( java.sql.Connection con, String tablePrefix ) 
    {
        this.tablePrefix = tablePrefix;
        this.con = con;        
        initComponents();
        showGui( 0, 0 ); // show top-left Editor Cell
    }
    //--------------------------------------------------------------------------
    // mainSplitPane
    //  
    //
    //      --------------------------------------------
    //      editSplitPane
    //          characterEditorPane  characterStatePane 
    //
    //--------------------------------------------------------------------------    
    private void initComponents() 
    {
        characterEditorPane     = new CharacterStateEditor();
        characterStatesPane     = new CharacterStateSelector();
        
        setLayout(new java.awt.BorderLayout());
        //-----------
        // Workframe
        //-----------        
        editSplitPane = new javax.swing.JSplitPane( javax.swing.JSplitPane.HORIZONTAL_SPLIT );
        editSplitPane.setOneTouchExpandable( true );
        // editSplitPane.setOrientation( javax.swing.JSplitPane.HORIZONTAL_SPLIT );
        // editSplitPane.setResizeWeight( 0.5 );
        // characterEditorPane.setMaximumSize( new java.awt.Dimension( 400, 100 ) );
        // characterEditorPane.setPreferredSize( new java.awt.Dimension( 400, 100 ) );
        // characterStatesPane.setMinimumSize( new java.awt.Dimension( 400, 100 ) );
        
        editSplitPane.setLeftComponent( characterEditorPane );
        editSplitPane.setRightComponent( characterStatesPane );        
        
        mainSplitPane           = new javax.swing.JSplitPane( javax.swing.JSplitPane.VERTICAL_SPLIT );
        mainSplitPane.setOneTouchExpandable( true );
        // mainSplitPane.setOrientation( javax.swing.JSplitPane.VERTICAL_SPLIT );
        mainSplitPane.setResizeWeight( 0.5 );
        
        mainSplitPane.setBottomComponent( editSplitPane );       
        //-------------
        // init Editor
        //-------------
        ded = new DeltaEditorData( con, tablePrefix );        
        // show IID and CID in TableHeaders
        
        ded.showHeaderIndex( true );
        ded.showItemIndex( true );
        DeltaEditorTableModel model = new DeltaEditorTableModel( ded );               
                
        // Create a column model for editior table
        javax.swing.table.TableColumnModel editorColumnModel = new javax.swing.table.DefaultTableColumnModel()
        {
            boolean first = true;
            
            public void addColumn( javax.swing.table.TableColumn tc )
            {
                // don' show fist column, first column is used as row header
                if( first )
                {
                    first = false;
                    return;
                }
                tc.setMinWidth( COLUMN_MIN_WIDTH );
                tc.setHeaderRenderer( new DeltaEditorTableCellRenderer() );
                super.addColumn( tc );
            }            
        };
        // editorColumnModel.setColumnSelectionAllowed( true );
        // Create a column model for the first table row (TAXON NAME)
        javax.swing.table.TableColumnModel rowHeaderColumnModel = new javax.swing.table.DefaultTableColumnModel()
        {
            boolean first = true;
            
            public void addColumn( javax.swing.table.TableColumn tc )
            {
                if( first )
                {
                    tc.setMinWidth( ROW_HEADER_MIN_WIDTH );
                    super.addColumn( tc );
                    first = false;
                }
            }
        };

        class DnTable extends javax.swing.JTable implements javax.swing.Scrollable 
        {
            public DnTable( javax.swing.table.TableModel model, javax.swing.table.TableColumnModel editorColumnModel )
            {
                super( model, editorColumnModel );        
            }
            /* (non-Javadoc)
             * @see javax.swing.Scrollable#getScrollableTracksViewportHeight()
             */
            public boolean getScrollableTracksViewportHeight() 
            {
                return super.getScrollableTracksViewportHeight(); 
                // return false;
            }

            /* (non-Javadoc)
             * @see javax.swing.Scrollable#getScrollableTracksViewportWidth()
             */
            public boolean getScrollableTracksViewportWidth() 
            {
                return super.getScrollableTracksViewportWidth();
                    // return false;
            }

            /* (non-Javadoc)
             * @see javax.swing.Scrollable#getPreferredScrollableViewportSize()
             */
            public Dimension getPreferredScrollableViewportSize() 
            {
                return super.getPreferredScrollableViewportSize();
                // return null;
            }

            /* (non-Javadoc)
             * @see javax.swing.Scrollable#getScrollableBlockIncrement(java.awt.Rectangle, int, int)
             */
            public int getScrollableBlockIncrement( Rectangle arg0, int arg1, int arg2 ) 
            {
                System.out.println( "getScrollableBlockIncrement" + arg0 + " " + arg1 + " " + arg2 );  
                return super.getScrollableBlockIncrement( arg0, arg1, arg2 );
                // return 0;
            }

            /* (non-Javadoc)
             * @see javax.swing.Scrollable#getScrollableUnitIncrement(java.awt.Rectangle, int, int)
             */
            public int getScrollableUnitIncrement(Rectangle arg0, int arg1, int arg2) 
            {
                System.out.println( "getScrollableUnitIncrement" + arg0 + " " + arg1 + " " + arg2 );  
                return super.getScrollableUnitIncrement( arg0, arg1, arg2 ); 
                // return 0;
            }            
        };
        // editorTable
        // ??? editorTable = new javax.swing.JTable( model, editorColumnModel );        
        editorTable = new DnTable( model, editorColumnModel );        
        editorTable.createDefaultColumnsFromModel();
        // Row Header Table
        rowHeaderTable = new javax.swing.JTable( model, rowHeaderColumnModel );
        rowHeaderTable.createDefaultColumnsFromModel();
        // Make sure thet selection between the editor an header stay in sync
        editorTable.setSelectionModel( rowHeaderTable.getSelectionModel() );
        editorTable.setCellSelectionEnabled( true );        
        // set Row Header look
        rowHeaderTable.setMaximumSize( new java.awt.Dimension( ROW_HEADER_MIN_WIDTH, 1000 ) );
        rowHeaderTable.setCellSelectionEnabled( false );        
        rowHeaderTable.setColumnSelectionAllowed( false );
        /*
        rowHeaderTable.setSelectionBackground( java.awt.Color.lightGray );
         */
        //
        rowHeaderTable.setCellSelectionEnabled( false );
        javax.swing.JViewport rhvp = new javax.swing.JViewport();
        rhvp.setView( rowHeaderTable );
        rhvp.setPreferredSize( new java.awt.Dimension( rowHeaderTable.getMaximumSize() ) );
        rowHeaderTable.setAutoResizeMode( javax.swing.JTable.AUTO_RESIZE_OFF );
        // editorTable.setRowHeight( 0, 80 );
//        editorTable.setDefaultRenderer( java.lang.String.class, new DeltaEditorTableCellRenderer() );
        editorTable.setDefaultRenderer( CharacterStateDescription.class, new DeltaEditorTableCellRenderer() );
        editorTable.setAutoResizeMode( javax.swing.JTable.AUTO_RESIZE_OFF );
        
        // editorTableScrollPane = new javax.swing.JScrollPane( editorTable );
        editorTableScrollPane = new javax.swing.JScrollPane();
        editorTableScrollPane.setViewportView( editorTable );
        editorTableScrollPane.setRowHeader( rhvp );
        
        /// editorTableScrollPane.getVerticalScrollBar().setUnitIncrement(10);
        editorTableScrollPane.getHorizontalScrollBar().setUnitIncrement( COLUMN_MIN_WIDTH + editorColumnModel.getColumnMargin() );
        /*
        editorTableScrollPane.getHorizontalScrollBar().addAdjustmentListener( new java.awt.event.AdjustmentListener()
        {
            public void adjustmentValueChanged(java.awt.event.AdjustmentEvent evt )
            {
                System.out.println( evt );
            }
        });
        */
        // add cell-editor to TOP
        mainSplitPane.setTopComponent( editorTableScrollPane );
        //
        add( mainSplitPane, java.awt.BorderLayout.CENTER );        
        //
        editSplitPane.setDividerLocation( this.getWidth() - SELECTION_WIDTH );        
        
        // Edit selected Tabel cell
        editorTable.addMouseListener(new java.awt.event.MouseAdapter()
        {
            public void mouseClicked( java.awt.event.MouseEvent evt )
            {
                editorTableMouseClicked( evt );
            }
        });
        
        editorTable.addKeyListener( new java.awt.event.KeyListener()
        {
            public void keyTyped( java.awt.event.KeyEvent evt )
            {
            }
            public void keyReleased( java.awt.event.KeyEvent evt )
            {
                // if( evt.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER )
                {
                    int row = editorTable.getSelectedRow();
                    int col = editorTable.getSelectedColumn();
                    showGui( row, col );                
                }
            }
            public void keyPressed( java.awt.event.KeyEvent evt )
            {
                // is invoked befor table selection
                // System.out.println( evt.getKeyCode() );
                // System.out.println( evt.getKeyChar() );                
            }            
        });        
        // Add new Table Item
        rowHeaderTable.addMouseListener(new java.awt.event.MouseAdapter()
        {
            /*
            public void mouseClicked( java.awt.event.MouseEvent evt )
            {
                 rowHeaderMouseClicked( evt );
            }
            */
            public void mousePressed( java.awt.event.MouseEvent evt ) 
            {
                // logger.debug( "mousePressed( " + e.getModifiers() + " )" );
                // Apple Mac used (ALT) ??? mouse for right butten simulation
                if( ( evt.getModifiers() & java.awt.event.InputEvent.BUTTON3_MASK ) == java.awt.event.InputEvent.BUTTON3_MASK ||
                    ( evt.getModifiers() & java.awt.event.InputEvent.BUTTON2_MASK ) == java.awt.event.InputEvent.BUTTON2_MASK   
                  )
                    
                {    
                     rowHeaderMouseClicked( evt );
                    // rightClick( evt.getX(), evt.getY() );
                }
            }

        });
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------               
    private void editorTableMouseClicked(java.awt.event.MouseEvent evt)
    {
        int row = editorTable.getSelectedRow();
        int col = editorTable.getSelectedColumn();
                
        logger.debug(  "MouseClicked row: " + row + " Col: " + col );
        showGui( row, col );                
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------               
    private void rowHeaderMouseClicked(java.awt.event.MouseEvent evt)
    {
        final int row = editorTable.getSelectedRow();
        final int col = editorTable.getSelectedColumn();                
        final javax.swing.JPanel deltaEditorPanel = this;
        
        logger.debug(  "RowHeader MouseClicked row: " + row + " Col: " + col );
                
        // create and show popup menu for new Item
        javax.swing.JPopupMenu addItemPopUp = new javax.swing.JPopupMenu();
        
        DnMenuItem  addNewItem  = new DnMenuItem( "delta_editor.add_new" );        
        DnMenuItem  addCopyItem = new DnMenuItem( "delta_editor.add_copy" );
        DnMenuItem  renameItem  = new DnMenuItem( "delta_editor.rename_item" );
        DnMenuItem  deleteItem  = new DnMenuItem( "delta_editor.delete_item" );
        DnMenuItem  gotoItem    = new DnMenuItem( "delta_editor.goto_item" );

        addNewItem.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
                // logger.debug( "DeltaEditor: insert new item " ); 
                java.lang.String itemName = DnOptionPane.DnShowInputDialog( deltaEditorPanel, "Input a new ItemName", "Insert Itemname", DnOptionPane.QUESTION_MESSAGE );                
                if( itemName == null )
                {
                    return;
                }
                else
                {
                    ded.insertNewItem( itemName );
                }
            }
        });
        // add a copy of a selected item
        addCopyItem.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
                // logger.debug( "DeletaEditor: insert copy item " );                 
                java.lang.String itemName = DnOptionPane.DnShowInputDialog( deltaEditorPanel, "Input a new ItemName \nCopy ItemData from \"" + ded.getItemText( row ) + "\"", "Copy and Insert Itemname", DnOptionPane.QUESTION_MESSAGE );                
                if( itemName == null )
                {
                    return;
                }
                else
                {
                    ded.insertCopyNewItem( itemName, ded.getIID( row ) );
                }
            }
        });                
        // delete item
        deleteItem.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
                // logger.debug( "DeletaEditor: insert copy item " );                 
                int i = DnOptionPane.DnShowConfirmDialog( deltaEditorPanel, "Delete this ItemName \nand all Item-Data\"" + ded.getItemText( row ) + "\"", "Delete Itemname and data", DnOptionPane.YES_NO_OPTION, DnOptionPane.QUESTION_MESSAGE );                
                if( i != DnOptionPane.YES_OPTION )
                {
                    return;
                }
                else
                {
                    ded.deleteItem( ded.getItemText( row ) );
                }
            }
        });                
        // rename itenName
        renameItem.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
                // logger.debug( "DeletaEditor: insert copy item " );                 
                java.lang.String itemName = DnOptionPane.DnShowInputDialog( deltaEditorPanel, "Rename this ItemName \nold name: \"" + ded.getItemText( row ) + "\"", "Rename Itemname", DnOptionPane.QUESTION_MESSAGE );                
                if( itemName == null )
                {
                    return;
                }
                else
                {
                    ded.renameItem( itemName, ded.getItemText( row ) );
                }
            }
        });
        // Goto item
        gotoItem.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
                // logger.debug( "DeletaEditor: goto item " );                 
                java.lang.String itemName = DnOptionPane.DnShowInputDialog( deltaEditorPanel, "Pleas select ItemName: ", "Goto ItemName", DnOptionPane.QUESTION_MESSAGE );                
                if( itemName == null )
                {
                    return;
                }
                else
                {
                    //search ItemName
                    int index = ded.getRowByItemName( itemName );
                    System.out.println( "Goto index: " + index );
                    if( index >= 0 )
                    {
                        editorTable.changeSelection( index,  0, false, false );
                    }
                }
            }
        });                
        
        addItemPopUp.add( addNewItem  );
        addItemPopUp.add( addCopyItem );
        addItemPopUp.add( renameItem  );
        addItemPopUp.add( deleteItem );
        addItemPopUp.add( gotoItem );

        int xx = evt.getX();
        int yy = evt.getY();
        yy += 80;   // Header size
	int x = xx; // - editorTableScrollPane.getHorizontalScrollBar().getValue();
	int y = yy - editorTableScrollPane.getVerticalScrollBar().getValue();
        
        addItemPopUp.show( this, x, y );
        
        // showGui( row, col );                
    }
    //--------------------------------------------------------------------------
    // Display GUI for selected row and collumn
    //--------------------------------------------------------------------------
    public void showGui( int row, int col )
    {
        // setup (new) CharacterSelector for this Entry        
        characterStatesPane = new CharacterStateSelector( ded.getHeadText( col ), ded.getTypeUnit( col ) );                
        characterStatesPane.setCharacterStates( ded.getCharacterStatesNames( col ) );
        // select character states for this CID
        // logger.debug( "CharacterState: " + (String) editorTable.getValueAt( row, col ) );
        CharacterStateDescription csd = ( CharacterStateDescription ) editorTable.getValueAt( row, col );
        characterStatesPane.setCharacterStatesDescription( csd.getDescriptionStatesVector() );
        editSplitPane.setRightComponent( characterStatesPane );
        //------------
        // Editor
        //------------
        characterEditorPane = new CharacterStateEditor( ded.getHeadText( col ) );                
        characterEditorPane.setCharacterStatesDescription( csd.getDescriptionStatesVector() );
        // Set Size for character-state-edit-area
        java.awt.Dimension editSize = new java.awt.Dimension();

        if( this.getWidth() - SELECTION_WIDTH < 200 )
        {
            editSize.setSize( 400, 150 );
        }
        else
        {
            editSize.setSize( this.getWidth() - SELECTION_WIDTH, 150 );
        }
        // characterEditorPane.setPreferredSize( new java.awt.Dimension( 400,200 ) );
        // characterEditorPane.setMinimumSize( new java.awt.Dimension( 400,200 ) );
        
        characterEditorPane.setPreferredSize( editSize );
        characterEditorPane.setMinimumSize( editSize );
        editSplitPane.setLeftComponent( characterEditorPane );
        //        
        editSplitPane.setDividerLocation( this.getWidth() - SELECTION_WIDTH );
        // is CharacterState editable ?
        if( csd.isInapplicalible() )            
        {
            characterEditorPane.setEditable( false );
        }
        else
        {
            characterEditorPane.setEditable( true );
        }
        //-------------------------------
        // Add listeners
        //-------------------------------
        deltaEditorChangeListener = new DeltaEditorChangeListener( characterEditorPane, ded, ded.getIID( row ), ded.getCID( col) ); 
        
        characterStatesPane.addDeltaEditorChangeListener( deltaEditorChangeListener );
        characterEditorPane.addDeltaEditorChangeListener( deltaEditorChangeListener );
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------               
    public static void main(String args[]) 
    {
        try
        {
            org.apache.log4j.BasicConfigurator.configure();
            logger.getRoot().setLevel( Level.DEBUG );
            logger.setLevel( Level.DEBUG );

            java.sql.Connection con = null;
            
            String db_url   = "jdbc:postgresql://hongkong:5432/DiversityDescriptions_LIASlight";
            String db_user  = "didi";
            String db_pwd   = "";
            
            // load driver and get DB-Connection
            try
            {
                Class.forName( "org.postgresql.Driver" );
                System.out.println( "Load PostgreSQL Driver..." );
            }
            catch( java.lang.ClassNotFoundException ee )
            {
                System.out.println( "ClassNotFoundException: " );
                System.out.println( ee.getMessage() );
            }
            try
            {
                con = java.sql.DriverManager.getConnection( db_url, db_user, db_pwd );
            }            
            catch( java.sql.SQLException ex )
            {
                System.out.println( ex.getMessage() );             
            }
    
    
            javax.swing.JFrame frame = new javax.swing.JFrame();
            DeltaEditor panel = new DeltaEditor( con, "LIASlight" );
            
            frame.setSize( 800, 600 );
            frame.setDefaultCloseOperation( javax.swing.JFrame.EXIT_ON_CLOSE );
            frame.getContentPane().add( panel );
            
            frame.show();
        }
        /*
        catch( java.sql.SQLException ex )
        {
            ex.printStackTrace();
        }
         */
        finally
        {
            
        }                
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------    
    public void actionPerformed(java.awt.event.ActionEvent actionEvent)
    {
        logger.debug( actionEvent );
    }
    
}
