package net.metadiversity.diversity.navigator.project;

import java.util.*;
import javax.xml.parsers.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;

import java.io.File;

import net.metadiversity.diversity.navigator.io.*;

import org.apache.log4j.Logger;

/*  all informations about projects are stored in an xml-file "project.xml"
 *  located in user-home/.diversity_navigator/ directory
 *
 */

public class ProjectManager 
{
    static Logger logger = Logger.getLogger( ProjectManager.class.getName() );
    static String configFileName;

    private static LinkedHashMap projects;
    
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public ProjectManager()
    {
        // read file from user.home directory
	String path = System.getProperty( "user.home" );
        configFileName = path + "/.diversity_navigator/diversity_navigator.connections.xml";
	projects = new LinkedHashMap();                
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public static ProjectInfo getProjectInfoByName( String projectName )
    {
        ProjectInfo pi = null;
        try
        {
            if( projects == null )
            {
                getProjects();
            }
            pi = ( ProjectInfo ) projects.get( projectName );
        }
        catch( java.lang.Exception ex )        
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );     
        }
        return pi;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public static LinkedHashMap getProjects() throws Exception 
    {
        // read file from user.home directory
        if( configFileName == null )
        {       
            String path = System.getProperty( "user.home" );
            configFileName = path + "/.diversity_navigator/diversity_navigator.projects.xml";
        }    
        // test if connextion-file exist        
	File f = new java.io.File( configFileName );
	if( !f.exists() ) 
        {
             logger.error( "Can't read file: " + configFileName ); 
             new LinkedHashMap();
        }
	projects = new LinkedHashMap();                
        // get XML factory
        try
        {
            javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            factory.setValidating( true );        // enable Validating with DTD
            javax.xml.parsers.DocumentBuilder docBuilder = factory.newDocumentBuilder();
            docBuilder.setErrorHandler( new XmlSaxErrorHandler() ); 
            org.w3c.dom.Document doc = docBuilder.parse( configFileName );
            Element root = doc.getDocumentElement();
            NodeList nodeList = root.getElementsByTagName( "project" );
            for( int i = 0; i < nodeList.getLength(); i++ ) 
            {
                Element projectNode = ( Element ) nodeList.item( i );
                String name = "";
                String description = "";
                String classname = "";
                String dbgroup ="";
                
                name        = ( (Element) projectNode.getElementsByTagName( "name" ).item(0)).getFirstChild().getNodeValue();
                
                description = ( (Element) projectNode.getElementsByTagName( "description" ).item(0)).getFirstChild().getNodeValue();
                classname   = ( (Element) projectNode.getElementsByTagName( "class" ).item(0)).getFirstChild().getNodeValue();
                projects.put( name, new ProjectInfo( name, description, classname ) );
                
                dbgroup     = ( (Element) projectNode.getElementsByTagName( "dbgroup" ).item(0)).getFirstChild().getNodeValue();
            }
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
	return projects;
    }
    //--------------------------------------------------------------------------
    // Add Project 
    //--------------------------------------------------------------------------
    public static void addProject( String name, String description, String classname ) 
    {
        try
        {
            LinkedHashMap projects = getProjects();
            projects.put( name, new ProjectInfo( name, description, classname ) );
            saveProjects( projects );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.toString() );
        }
    }
    //--------------------------------------------------------------------------
    // Change Project 
    //--------------------------------------------------------------------------
    public static void changeProject( String name, String description, String classname )
    {
        try
        {
            LinkedHashMap projects = getProjects();
            projects.remove( name );
            projects.put( name, new ProjectInfo( name, description, classname ) );
            saveProjects( projects );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.toString() );
        }
    }
    //--------------------------------------------------------------------------
    // Delete Project 
    //--------------------------------------------------------------------------
    public static void deleteProject( String name )    
    {
        try
        {
            LinkedHashMap projects = getProjects();
            projects.remove( name );
            saveProjects( projects );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.toString() );
        }
    }
    //--------------------------------------------------------------------------
    // Save Project 
    //--------------------------------------------------------------------------
    private static void saveProjects( LinkedHashMap projects )
    {
        try
        {
            logger.info( "Write file: " + configFileName );
            javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder docBuilder = factory.newDocumentBuilder();
            docBuilder.setErrorHandler( new XmlSaxErrorHandler() ); 
            
            factory.setValidating( true );        // enable Validating with DTD
                       
            org.w3c.dom.Document doc = docBuilder.newDocument();
            Element rootElement     = doc.createElement( "PROJECTS" );
            
            doc.appendChild( rootElement );

            Iterator iter = projects.keySet().iterator();
            while( iter.hasNext() )
            {
                ProjectInfo pi = ( ProjectInfo ) projects.get( iter.next() );
            
                Element projectElement      = doc.createElement( "project" );
                Element nameElement         = doc.createElement( "name" );
                Element descriptionElement  = doc.createElement( "description" );
                Element dbgroupElement      = doc.createElement( "dbgroup" );
                Element classElement        = doc.createElement( "class" );
            
                rootElement.appendChild( projectElement );

                projectElement.appendChild( nameElement );    
                projectElement.appendChild( descriptionElement );    
                projectElement.appendChild( dbgroupElement );    
                projectElement.appendChild( classElement );    


                nameElement.appendChild(        doc.createTextNode( pi.getName() ) );
                descriptionElement.appendChild( doc.createTextNode( pi.getDescription() ) );
                // not implemented
                dbgroupElement.appendChild(     doc.createTextNode( "not implementet" ) );
            }
            
            
            javax.xml.transform.Transformer t = javax.xml.transform.TransformerFactory.newInstance().newTransformer();
            //
            // Properties see: javax.xml.transform.OutputKeys
            //
            t.setOutputProperty( "method", "xml" );
            t.setOutputProperty( "standalone" , "no" );
            t.setOutputProperty( "indent" , "yes" );
            t.setOutputProperty( "doctype-system" , "diversity_navigator.projects.dtd" );
            // write to file
            t.transform( new javax.xml.transform.dom.DOMSource( doc ),  new javax.xml.transform.stream.StreamResult( new java.io.FileOutputStream( configFileName ) ));                                    
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }            
    }
    //--------------------------------------------------------------------------
    // Errorhandler
    //--------------------------------------------------------------------------
    public static class XmlSaxErrorHandler implements org.xml.sax.ErrorHandler
    {        
        
        public void warning( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.warn( buildMsg( ex ) );
        }
        public void error( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.error( buildMsg( ex ) );
        }
        public void fatalError( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException
        {
            // System.out.println( exception.toString() );
            logger.fatal( buildMsg( ex ) );            
        }
        // format error message
        private String buildMsg( org.xml.sax.SAXParseException ex )
        {
            String msg = configFileName + "\n Line: " + ex.getLineNumber() + " Column: " + ex.getColumnNumber() + "\n" + ex.toString();            
            return msg;
        }
    }
}
